#version 330 core

in vec2 fTexcoord;
in vec3 fNormals;
in vec3 fPosition;
in vec4 shadowCoord1;
in vec4 shadowCoord2;
in vec4 shadowCoord3;

struct SpotLight {
	vec3 position;
	vec3 intensity;
	vec3 direction;
	float angle;
};

out vec4 outColor;

uniform mat4 model;
uniform sampler2D shadowMap1;
uniform sampler2D shadowMap2;
uniform sampler2D shadowMap3;
uniform sampler2D tex;
uniform mat4 depthBias;
uniform SpotLight spotLight1;
uniform SpotLight spotLight2;
uniform SpotLight spotLight3;

float calcShadow(vec4 shadowCoord, sampler2D shadowMap) {
	float bias = 0.0009;
	vec2 texSize = 1.0 / textureSize(shadowMap, 0);
	vec3 projCoord = shadowCoord.xyz / shadowCoord.w;
	float currentDepth = projCoord.z - bias;

	float shadow = 0;

	for (float x = -1; x <= 1; x += 1.0) {
		for (float y = -1; y <= 1; y += 1.0) {
			float pcfValue = texture(shadowMap, projCoord.xy + vec2(x, y) * texSize).r;
			shadow += currentDepth > pcfValue ? 1.0 : 0.0;  
		}
	}

	shadow = shadow / 9.0;
	return 1.0 - shadow;
}

vec3 calcLight(SpotLight spotLight, vec4 shadowCoord, sampler2D shadowMap) {
	vec3 lightPosition = spotLight.position;
	vec3 lightDirection = spotLight.direction;
	float lightCutOff = cos(radians(spotLight.angle));
	float lightCutOffInner = cos(radians(spotLight.angle - 5.0f));
	float lightDiffuse = 0.5f;
	float lightConstant = 0.5f;
	float lightLinear =  0.0014f;
	float lightQuadratic = 0.000007f;
	
    vec3 toLightDir = normalize(lightPosition - fPosition);
    float theta = dot(toLightDir, normalize(-lightDirection)); 
	float cutOffDiff = lightCutOffInner - lightCutOff;
	float intensity = clamp((theta - lightCutOff) / cutOffDiff, 0.0, 1.0);
	float shadow = calcShadow(shadowCoord, shadowMap);
    
    if(theta >= lightCutOff) {    
        // Diffuse 
        vec3 norm = normalize(fNormals);        
        float diff = max(dot(norm, toLightDir), 0.0);
        vec3 diffuse = vec3(lightDiffuse * diff);  
        
        // Attenuation
        float distance = length(lightPosition - fPosition);
        float attenuation = 1.0f / (lightConstant + lightLinear * distance + lightQuadratic * (distance * distance)); 
        diffuse = shadow * (diffuse * attenuation);
                
        return diffuse * intensity;  
    } else {
        return vec3(0.0f, 0.0f, 0.0f);
	}
}

void main(void) {
	vec3 ambient = vec3(0.4f, 0.4f, 0.4f);
	vec3 diffuse = calcLight(spotLight1, shadowCoord1, shadowMap1);
	diffuse += calcLight(spotLight2, shadowCoord2, shadowMap2);
	diffuse += calcLight(spotLight3, shadowCoord3, shadowMap3);
	vec4 textureColor = texture(tex, fTexcoord);
	outColor = vec4(vec4((ambient + diffuse), 1.0f) * textureColor);

}